//  ----------------------------------------------------------------------------
//			Inicializacia procesora na najnizsej urovni
//			- nastavenie hodin (oscilator)
//			- nastavenie preruseni
//  ----------------------------------------------------------------------------

// Hlavickovy subor procesora
#include "AT91SAM7S64.h"

// Nasledovne funkcie musia byt napisane v ARM mode
// Tieto funkcie su priamo volane pri obsluhe preruseni
extern void AT91F_Spurious_handler(void);
extern void AT91F_Default_IRQ_handler(void);
extern void AT91F_Default_FIQ_handler(void);

//*----------------------------------------------------------------------------
//* AT91F_LowLevelInit
//*----------------------------------------------------------------------------
void AT91F_LowLevelInit(void)
{	int i;
	AT91PS_PMC pPMC = AT91C_BASE_PMC;

	// Nastavenie casovania pristupu do Flash (Flash Wait state)
	//  Pristup v jednom cykle pre frekvencie do 30 MHz, alebo 40
	//  ak je MCK = 47923200 na 1 us pripada 48 cyklov
	AT91C_BASE_MC->MC_FMR = ((AT91C_MC_FMCN)&(48 <<16)) | AT91C_MC_FWS_1FWS;

	// Zakazanie Watchdogu
//	AT91C_BASE_WDTC->WDTC_WDMR = AT91C_WDTC_WDDIS;

	// Nastavenie MCK na 47 923 200 Hz
	// 1. Povolenie hlavneho oscilatora (Main Oscillator)
	// SCK (Slow Clock) = 1/32768 = 30.51 uSecond
	// Dlzka startu = 8 * 6 / SCK = 48 * 30.51 = 1,46484375 ms
	pPMC->PMC_MOR = (( AT91C_CKGR_OSCOUNT & (0x06 <<8) | AT91C_CKGR_MOSCEN ));
	
	// Cakanie na ukoncenie startu
	while(!(pPMC->PMC_SR & AT91C_PMC_MOSCS));

	// 2. Kontrola frekvencie hlavneho oscilatora (volitelne)
	// 3. Nastavenie PLL a delitela:
		// - delenie piatimi: Fin = 3,6864 = (18,432 / 5)
		// - nasobenie 25+1: Fout = 95,8464 = (3,6864 * 26)
		// vzhladom na 96 MHz je chyba 0.16%
		// PLLCOUNT dlzka startu PLL (odhad): 0.844 ms
		// PLLCOUNT 28 = 0.000844 /(1/32768)
		pPMC->PMC_PLLR = ((AT91C_CKGR_DIV & 0x05) |
						  (AT91C_CKGR_PLLCOUNT & (28<<8)) |
						  (AT91C_CKGR_MUL & (25<<16)));
		// Cakanie na ukoncenie startu
		while(!(pPMC->PMC_SR & AT91C_PMC_LOCK));
		while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

	// 4. Nastavenie Master Clock a Processor Clock
	// PLL clock delene dvomi
		pPMC->PMC_MCKR =  AT91C_PMC_PRES_CLK_2 ;
		while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

		pPMC->PMC_MCKR |= AT91C_PMC_CSS_PLL_CLK  ;
		while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

	// Nastavenie obsluhy preruseni
	AT91C_BASE_AIC->AIC_SVR[0] = (int) AT91F_Default_FIQ_handler;
	for (i=1;i<31;i++)
	{	AT91C_BASE_AIC->AIC_SVR[i] = (int) AT91F_Default_IRQ_handler ;
	}
	AT91C_BASE_AIC->AIC_SPU  = (int) AT91F_Spurious_handler ;
}
